package main

import (
	"strconv"
	"sync"
	"time"

	"github.com/go-vgo/robotgo"
	"fyne.io/fyne/v2"
	"fyne.io/fyne/v2/app"
	"fyne.io/fyne/v2/container"
	"fyne.io/fyne/v2/driver/desktop"
	"fyne.io/fyne/v2/widget"
)

var (
	clicking = false
	lock     = sync.RWMutex{} // Protects the `clicking` variable from concurrent access
)

func clickInterval(quit chan struct{}, interval time.Duration) {
	time.Sleep(1 * time.Second) // Delay before start clicking
	for {
		select {
		case <-quit:
			return
		default:
			// Perform the click
			robotgo.Click()

			// Sleep for the interval before the next click
			time.Sleep(interval)
		}
	}
}

func main() {
	// Create a new application
	myApp := app.New()

	// Create a new window for the application
	myWindow := myApp.NewWindow("Auto Clicker")
	myWindow.Resize(fyne.NewSize(300, 90)) // Resize the window

	// Add shortcut to close window on CTRL+Q
	myWindow.Canvas().AddShortcut(&desktop.CustomShortcut{KeyName: fyne.KeyQ, Modifier: desktop.ControlModifier}, func(shortcut fyne.Shortcut) {
		myApp.Quit()
	})

	// Create a button widget with "Start Clicking" label
	button := widget.NewButton("Start Clicking", nil)

	// Create text entry widgets to input the interval between clicks
	entrySeconds := widget.NewEntry()
	entrySeconds.PlaceHolder = "Seconds"
	entryMilliseconds := widget.NewEntry()
	entryMilliseconds.PlaceHolder = "Milliseconds"

	var quit chan struct{} // Channel to signal the clicker goroutine to stop

	// Set the button action based on the current state
	button.OnTapped = func() {
		lock.Lock() // Lock to safely update the `clicking` variable
		defer lock.Unlock()

		if clicking {
			// Stop clicking
			close(quit) // This will cause the clicker goroutine to return
			clicking = false
			button.SetText("Start Clicking")
		} else {
			// Do not start clicking if both fields are empty
			if entrySeconds.Text == "" && entryMilliseconds.Text == "" {
				return
			}

			// Check if the inputs are valid (i.e., empty or integers)
			seconds, errSec := strconv.Atoi(entrySeconds.Text)
			milliseconds, errMilli := strconv.Atoi(entryMilliseconds.Text)

			if (errSec != nil && entrySeconds.Text != "") || (errMilli != nil && entryMilliseconds.Text != "") {
				return
			}

			// Start clicking
			quit = make(chan struct{})
			interval := time.Duration(seconds)*time.Second + time.Duration(milliseconds)*time.Millisecond
			go clickInterval(quit, interval)
			clicking = true
			button.SetText("Stop Clicking")
		}
	}

	// Add the entry and button to the window and show it
	myWindow.SetContent(
		container.NewVBox(
			container.NewGridWrap(fyne.NewSize(141, 34), entrySeconds, entryMilliseconds),
			button,
		),
	)
	myWindow.ShowAndRun()
}
